package com.example.catchTheLetters.service.impl;

import com.example.catchTheLetters.constant.CommonConstant;
import com.example.catchTheLetters.constant.HttpStatus;
import com.example.catchTheLetters.model.dto.LoginDto;
import com.example.catchTheLetters.model.dto.RegisterDto;
import com.example.catchTheLetters.entity.User;
import com.example.catchTheLetters.model.vo.UserVo;
import com.example.catchTheLetters.service.AuthService;
import com.example.catchTheLetters.service.RedisService;
import com.example.catchTheLetters.utils.JwtUtil;
import com.example.catchTheLetters.utils.R;
import jakarta.annotation.Resource;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.regex.Pattern;

@Service
public class AuthServiceImpl implements AuthService {

    @Resource
    private MongoTemplate mongoTemplate;

    @Resource
    private RedisService redisService;

    @Override
    public R<String> login(LoginDto loginDto) {
        // 根据用户名查询用户信息
        var username = loginDto.getUsername();
        var matchType = getMatchType(username);

        var user = switch (matchType) {
            case CommonConstant.EMAIL ->
                    mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.EMAIL).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
            case CommonConstant.PHONE ->
                    mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.PHONE).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
            default ->
                    mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.USERNAME).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
        };

        if (Objects.isNull(user)) {
            return R.fail("用户不存在");
        }
        if (!user.getPassword().equals(loginDto.getPassword())) {
            return R.fail("密码错误");
        }
        Map<String, String> map = new HashMap<>();
        map.put("id", user.getId());
        var token = JwtUtil.getToken(map, CommonConstant.TOKEN_EXPIRE_TIME);
        redisService.hSet(token, user.getId(), CommonConstant.REDIS_EXPIRE_TIME);
        return R.ok(token);
    }

    @Override
    public boolean logout(String token) {
        return redisService.del(token);
    }

    @Override
    public User verify(String token) {
        var map = JwtUtil.getPayload(token);
        String id = null;
        if (map != null) {
            id = map.get("id").asString().replaceAll("\"", "");
        }
        var user = mongoTemplate.findOne(new Query(Criteria.where("id").is(id).and(CommonConstant.STATUS).ne(0)), User.class);
        if (Objects.isNull(user)) {
            return null;
        }
        return user;
    }

    @Override
    public R<UserVo> register(RegisterDto registerDto) {
        // 根据用户名查询用户信息
        var username = registerDto.getUsername();
        var matchType = getMatchType(username);

        User user;
        switch (matchType) {
            case CommonConstant.EMAIL -> {
                user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.EMAIL).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
                if (user != null) {
                    return R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
                }
                user = new User();
                user.setEmail(username);
            }
            case CommonConstant.PHONE -> {
                user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.PHONE).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
                if (user != null) {
                    return R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
                }
                user = new User();
                user.setPhone(username);
            }
            default -> {
                user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.USERNAME).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
                if (user != null) {
                    return R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
                }
                user = new User();
                user.setUsername(username);
            }
        }

        user.setPassword(registerDto.getPassword());
        user.setCreateTime(System.currentTimeMillis());
        user.setUpdateTime(System.currentTimeMillis());
        var insert = mongoTemplate.insert(user);
        if (insert.getId() != null) {
            return R.ok(user.toVo());
        } else {
            return R.fail("注册失败,请重试");
        }
    }

    /**
     * 获取用户名匹配类型（通常是登录和注册时使用）
     *
     * @param username 用户名
     * @return 匹配类型
     */
    private String getMatchType(String username) {
        var patternEmail = Pattern.compile(CommonConstant.EMAIL_REGEX);
        var matcherEmail = patternEmail.matcher(username);
        if (matcherEmail.matches()) {
            return CommonConstant.EMAIL;
        }
        var patternPhone = Pattern.compile(CommonConstant.PHONE_REGEX);
        var matcherPhone = patternPhone.matcher(username);
        if (matcherPhone.matches()) {
            return CommonConstant.PHONE;
        }
        return CommonConstant.USERNAME;
    }

    @Override
    public User getUserInfo(String id) {
        return mongoTemplate.findOne(new Query(Criteria.where("id").is(id).and(CommonConstant.STATUS).ne(0)), User.class);
    }

    @Override
    public User getUserByName(String username) {
        return mongoTemplate.findOne(new Query(Criteria.where("username").is(username).and(CommonConstant.STATUS).ne(0)), User.class);
    }

    @Override
    public R<UserVo> update(User user, String token) {
        // 先获取当前账户，比对id是否一致，然后再插入
        var verify = verify(token);
        if (verify == null || !verify.getId().equals(user.getId())) {
            return R.fail(HttpStatus.TOKEN_EXCEPTION.getCode(), HttpStatus.TOKEN_EXCEPTION.getDesc());
        }

        // 如果用户名修改了，检查是否有重复，有重复则返回null，否则修改
        if (user.getUsername() != null && (verify.getUsername() == null || !verify.getUsername().equals(user.getUsername()))) {
            var regedUser = getUserByName(user.getUsername());
            if (regedUser != null) {
                return R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
            }

            verify.setUsername(user.getUsername());
        }

        // 如果邮箱修改了，检查是否有重复，有重复则返回null，否则修改
        if (user.getEmail() != null && (verify.getEmail() == null || !verify.getEmail().equals(user.getEmail()))) {
            var regedUser = mongoTemplate.findOne(new Query(Criteria.where("email").is(user.getEmail()).and(CommonConstant.STATUS).ne(0)), User.class);
            if (regedUser != null) {
                return R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
            }

            verify.setEmail(user.getEmail());
        }

        // 如果手机号修改了，检查是否有重复，有重复则返回null，否则修改
        if (user.getPhone() != null && (verify.getPhone() == null || !verify.getPhone().equals(user.getPhone()))) {
            var regedUser = mongoTemplate.findOne(new Query(Criteria.where("phone").is(user.getPhone()).and(CommonConstant.STATUS).ne(0)), User.class);
            if (regedUser != null) {
                return R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
            }

            verify.setPhone(user.getPhone());
        }

        // 如果密码为空，意味着没修改密码，否则修改密码
        if (user.getPassword() != null && !user.getPassword().isEmpty()) {
            verify.setPassword(user.getPassword());
        }

        verify.setAvatar(user.getAvatar());
        verify.setSex(user.getSex());
        verify.setIntroduction(user.getIntroduction());
        verify.setUpdateTime(System.currentTimeMillis());

        return R.ok(mongoTemplate.save(verify).toVo());
    }
}
